import _extends from "@babel/runtime/helpers/esm/extends";
/* eslint-disable react/prop-types */
import { useEffect, useRef, useState } from 'react';
import cx from 'classnames';
import PropTypes from 'prop-types';
import AtomButton, { atomButtonSizes } from '@s-ui/react-atom-button';
import DropdownBasic from '@s-ui/react-dropdown-basic';
import DropdownUser from '@s-ui/react-dropdown-user';
import useIsomorphicLayoutEffect from '@s-ui/react-hooks/lib/useIsomorphicLayoutEffect/index.js';
import Menu from '@s-ui/react-icons/lib/Menu';
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
var noop = function noop() {};
var DEFAULT_NAV_WRAP_STYLE = {
  top: 'inherit',
  left: 'inherit',
  height: 'inherit',
  width: 'inherit'
};
var HTML_HAS_SCROLL_DISABLED = 'html-has-scroll-disabled';
var BODY_HAS_SCROLL_DISABLED = 'body-has-scroll-disabled';
var TITLE_CLASS_NAME = 'sui-TopbarUser-title';
var FLOW_BUTTON_CLASS_NAME = 'sui-TopbarUser-navButton';

/**
 * Render main navigation function.
 */
var renderNavMain = function renderNavMain(_ref) {
  var isToggleHidden = _ref.isToggleHidden,
    linkFactory = _ref.linkFactory;
  return function (_ref2, index) {
    var icon = _ref2.icon,
      text = _ref2.label,
      menu = _ref2.menu,
      arrowButtonIcon = _ref2.arrowButtonIcon,
      _ref2$onClick = _ref2.onClick,
      onClick = _ref2$onClick === void 0 ? noop : _ref2$onClick;
    var handleToggleMenu = function handleToggleMenu() {
      return onClick();
    };
    return /*#__PURE__*/_jsx(DropdownBasic, {
      button: {
        icon: icon,
        text: text,
        arrowButtonIcon: arrowButtonIcon
      },
      menu: menu,
      expandOnMouseOver: isToggleHidden,
      linkFactory: linkFactory,
      onToggleMenu: handleToggleMenu
    }, index);
  };
};

/**
 * Topbar containing a dropdown with user data (login, logout, secured links...).
 */
export default function TopbarUser(_ref3) {
  var brand = _ref3.brand,
    customContent = _ref3.customContent,
    _ref3$elementsToKeepS = _ref3.elementsToKeepScrollOnToggleMenu,
    elementsToKeepScrollOnToggleMenu = _ref3$elementsToKeepS === void 0 ? [] : _ref3$elementsToKeepS,
    _ref3$linkFactory = _ref3.linkFactory,
    linkFactory = _ref3$linkFactory === void 0 ? function (_ref4) {
      var href = _ref4.href,
        className = _ref4.className,
        children = _ref4.children,
        target = _ref4.target,
        title = _ref4.title;
      return /*#__PURE__*/_jsx("a", {
        href: href,
        className: className,
        target: target,
        title: title,
        children: children
      });
    } : _ref3$linkFactory,
    navButton = _ref3.navButton,
    navCTA = _ref3.navCTA,
    navCTALogin = _ref3.navCTALogin,
    navMain = _ref3.navMain,
    navUser = _ref3.navUser,
    _ref3$onToggle = _ref3.onToggle,
    onToggle = _ref3$onToggle === void 0 ? function () {} : _ref3$onToggle,
    _ref3$shouldDisplayNa = _ref3.shouldDisplayNavUser,
    shouldDisplayNavUser = _ref3$shouldDisplayNa === void 0 ? true : _ref3$shouldDisplayNa,
    _ref3$shouldDisplayTo = _ref3.shouldDisplayToggle,
    shouldDisplayToggle = _ref3$shouldDisplayTo === void 0 ? true : _ref3$shouldDisplayTo,
    _ref3$showBrandIcon = _ref3.showBrandIcon,
    showBrandIcon = _ref3$showBrandIcon === void 0 ? false : _ref3$showBrandIcon,
    title = _ref3.title,
    _ref3$toggleIcon = _ref3.toggleIcon,
    toggleIcon = _ref3$toggleIcon === void 0 ? Menu : _ref3$toggleIcon;
  var _topbarUserNode = useRef(null);
  var _topbarUserToggleNode = useRef(null);
  var _windowWidth = useRef();
  var _useState = useState(false),
    menuExpanded = _useState[0],
    setMenuExpanded = _useState[1];
  var _useState2 = useState(false),
    isToggleHidden = _useState2[0],
    setToggleHidden = _useState2[1];
  var _useState3 = useState(DEFAULT_NAV_WRAP_STYLE),
    navWrapStyle = _useState3[0],
    setNavWrapStyle = _useState3[1];

  /**
   * Set navigation wrap inline styles.
   */
  var _setNavWrapStyles = function _setNavWrapStyles() {
    var _topbarUserNode$curre = _topbarUserNode.current.getBoundingClientRect(),
      top = _topbarUserNode$curre.top,
      left = _topbarUserNode$curre.left,
      height = _topbarUserNode$curre.height,
      width = _topbarUserNode$curre.width;
    var navWrapTop = top + height;
    setNavWrapStyle({
      top: navWrapTop,
      left: left,
      height: window.innerHeight - navWrapTop,
      width: width
    });
  };

  /**
   * Toggle menu state: expanded/collapsed.
   */
  var _toggleMenu = function _toggleMenu() {
    var nextState = !menuExpanded;
    setMenuExpanded(nextState);
    onToggle({
      isOpen: nextState
    });
  };

  /**
   * Handle click on navigation wrap.
   */
  var _handleNavWrapClick = function _handleNavWrapClick(_ref5) {
    var target = _ref5.target,
      currentTarget = _ref5.currentTarget;
    if (menuExpanded && target === currentTarget) {
      _toggleMenu();
    }
  };
  useIsomorphicLayoutEffect(function () {
    // Early return if not on client
    if (window === undefined) return;
    /**
     * Set the display state for toggle button.
     */
    var _setToggleDisplayState = function _setToggleDisplayState() {
      if (!shouldDisplayToggle) return;
      // Only go on if user has been resized the browser window horizontally.
      if (window.innerWidth === _windowWidth.current) return;
      // Then save the new global value again.
      _windowWidth.current = window.innerWidth;
      var _window$getComputedSt = window.getComputedStyle(_topbarUserToggleNode.current),
        display = _window$getComputedSt.display;
      var isToggleCurrentlyHidden = display === 'none';
      if (!isToggleCurrentlyHidden) {
        _setNavWrapStyles();
      }
      if (isToggleCurrentlyHidden !== isToggleHidden) {
        setToggleHidden(isToggleCurrentlyHidden);
      }
    };
    _setToggleDisplayState();
    window.addEventListener('resize', _setToggleDisplayState);
    return function () {
      window.removeEventListener('resize', _setToggleDisplayState);
    };
  }, [isToggleHidden]);
  useEffect(function () {
    var _verticalScrollPosition;
    /**
     * Lock body element scroll.
     */
    var _lockBodyScroll = function _lockBodyScroll() {
      _verticalScrollPosition = window.scrollY;
      var transformStyleToKeepScroll = "translate3d(0, -" + _verticalScrollPosition + "px, 0)";
      window.document.documentElement.classList.add(HTML_HAS_SCROLL_DISABLED);
      window.document.body.classList.add(BODY_HAS_SCROLL_DISABLED);
      elementsToKeepScrollOnToggleMenu.forEach(function (selector) {
        document.querySelector(selector).style.transform = transformStyleToKeepScroll;
      });
    };

    /**
     * Unlock body element scroll.
     */
    var _unlockBodyScroll = function _unlockBodyScroll() {
      elementsToKeepScrollOnToggleMenu.forEach(function (selector) {
        var element = document.querySelector(selector);
        if (element) element.style.transform = '';
      });
      window.document.documentElement.classList.remove(HTML_HAS_SCROLL_DISABLED);
      window.document.body.classList.remove(BODY_HAS_SCROLL_DISABLED);
      elementsToKeepScrollOnToggleMenu.length && window.scrollTo(0, _verticalScrollPosition);
    };

    // Given toggle button is hidden in desktop.
    // Then we must not lock/unlock body there
    if (isToggleHidden) return;
    if (menuExpanded) {
      _lockBodyScroll();
    } else {
      _unlockBodyScroll();
    }
  }, [menuExpanded, isToggleHidden, elementsToKeepScrollOnToggleMenu]);
  var Link = linkFactory;
  var ToggleIcon = toggleIcon;
  var BrandIcon = brand.icon,
    BrandImage = brand.image,
    brandName = brand.name,
    brandUrl = brand.url;
  var avatar = navUser.avatar,
    name = navUser.name,
    menu = navUser.menu,
    hasUserBadgeLabel = navUser.hasUserBadgeLabel;
  var navWrapClassName = cx('sui-TopbarUser-navWrap', {
    'is-expanded': menuExpanded
  });
  var hasNotifications = navUser.menu.some(function (_ref6) {
    var notifications = _ref6.notifications;
    return Boolean(notifications);
  });
  var toggleMenuClassName = cx('sui-TopbarUser-toggle', {
    'has-notifications': hasNotifications
  });
  var handleCTAclick = navCTA && navCTA.onClick;
  var handleCTALoginClick = navCTALogin && navCTALogin.onClick;
  return /*#__PURE__*/_jsxs("div", {
    ref: _topbarUserNode,
    className: "sui-TopbarUser",
    children: [/*#__PURE__*/_jsxs("div", {
      className: "sui-TopbarUser-wrap",
      children: [navButton ? /*#__PURE__*/_jsx("div", {
        className: FLOW_BUTTON_CLASS_NAME,
        children: navButton
      }) : null, shouldDisplayToggle ? /*#__PURE__*/_jsx("button", {
        ref: _topbarUserToggleNode,
        className: toggleMenuClassName,
        onClick: _toggleMenu,
        children: /*#__PURE__*/_jsx(ToggleIcon, {
          svgClass: "sui-TopbarUser-toggleIcon"
        })
      }) : /*#__PURE__*/_jsx(_Fragment, {}), showBrandIcon ? /*#__PURE__*/_jsx(Link, {
        href: brandUrl,
        className: "sui-TopbarUser-brandIcon",
        title: brandName,
        children: /*#__PURE__*/_jsx(BrandIcon, {})
      }) : /*#__PURE__*/_jsx(Link, {
        href: brandUrl,
        className: "sui-TopbarUser-brand",
        title: brandName,
        children: BrandImage ? /*#__PURE__*/_jsx(BrandImage, {}) : brandName
      }), title ? /*#__PURE__*/_jsx("div", {
        className: TITLE_CLASS_NAME,
        children: title
      }) : null, /*#__PURE__*/_jsx("div", {
        className: navWrapClassName,
        style: isToggleHidden ? DEFAULT_NAV_WRAP_STYLE : navWrapStyle,
        onClick: _handleNavWrapClick,
        children: /*#__PURE__*/_jsxs("div", {
          className: "sui-TopbarUser-nav",
          children: [/*#__PURE__*/_jsx("div", {
            className: "sui-TopbarUser-navMain",
            children: navMain.map(renderNavMain({
              isToggleHidden: isToggleHidden,
              linkFactory: linkFactory
            }))
          }), shouldDisplayNavUser && /*#__PURE__*/_jsx("div", {
            className: "sui-TopbarUser-navUser",
            children: /*#__PURE__*/_jsx(DropdownUser, {
              user: {
                avatar: avatar,
                name: name
              },
              menu: menu,
              expandOnMouseOver: true,
              hasNotifications: hasNotifications && !menuExpanded,
              linkFactory: linkFactory,
              hasBadgeLabel: hasUserBadgeLabel
            })
          })]
        })
      })]
    }), customContent ? /*#__PURE__*/_jsx("div", {
      className: "sui-TopbarUser-customContent",
      children: customContent
    }) : /*#__PURE__*/_jsx(_Fragment, {}), /*#__PURE__*/_jsxs("div", {
      className: "sui-TopbarUser-cta",
      children: [navCTALogin && !customContent && /*#__PURE__*/_jsx("div", {
        className: "sui-TopbarUser-ctaAccess",
        children: /*#__PURE__*/_jsx(AtomButton, {
          link: true,
          linkFactory: linkFactory,
          design: 'outline',
          href: navCTALogin.url,
          title: navCTALogin.text,
          shape: navCTALogin.shape,
          size: atomButtonSizes.SMALL,
          onClick: handleCTALoginClick,
          children: navCTALogin.text
        })
      }), navCTA && !customContent && /*#__PURE__*/_jsx("div", {
        className: "sui-TopbarUser-ctaButton",
        children: /*#__PURE__*/_jsx(AtomButton, _extends({
          link: true,
          linkFactory: linkFactory,
          design: 'solid',
          href: navCTA.url,
          title: navCTA.text
        }, navCTA.icon && {
          leftIcon: /*#__PURE__*/_jsx(navCTA.icon, {
            svgClass: "sui-TopbarUser-ctaButtonIcon"
          })
        }, {
          shape: navCTA.shape,
          size: atomButtonSizes.SMALL,
          onClick: handleCTAclick,
          children: navCTA.text
        }))
      })]
    })]
  });
}
TopbarUser.displayName = 'TopbarUser';